<?php

namespace ByDedi\MediaBundle\Admin;

use ByDedi\AdminBundle\Admin\BaseAdmin;
use ByDedi\AdminBundle\Form\Type\TranslatableType;
use ByDedi\MediaBundle\Entity\Image;
use Sonata\AdminBundle\Datagrid\DatagridMapper;
use Sonata\AdminBundle\Datagrid\ListMapper;
use Sonata\AdminBundle\Form\FormMapper;
use Sonata\AdminBundle\Route\RouteCollection;
use Sonata\CoreBundle\Validator\ErrorElement;
use Symfony\Component\Form\Extension\Core\Type\FileType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\TextType;

class ImageAdmin extends BaseAdmin
{
    protected $datagridValues = array(
        '_page' => 1,
        '_sort_order' => 'DESC',
        '_sort_by' => 'id'
    );


    protected function configureDatagridFilters(DatagridMapper $datagridMapper)
    {
        $datagridMapper
            ->add('translations.name', null, ['label' => 'Titre'])
        ;
    }


    protected function configureListFields(ListMapper $listMapper)
    {
        $action = explode('::', $this->getRequest()->attributes->get('_controller'));
        if (!empty($action[1])) {
            $action = $action[1];
        }
        if ($action == 'browseAction') {
            $listMapper
                ->add('getPathFile', null, ['label' => 'Image',
                    'template' => $this->getTemplateType('image')
                ])
                ->add('name', null, ['label' => 'Titre','editable' => true])
                ->add('alt', null, ['label' => 'Alt','editable' => true])
                ->add('getDimensionsString', null, ['label' => 'Dimensions'])
                ->add('getWeightInMB', null, ['label' => 'Poids'])
                ->add('_action', 'actions', array(
                    'actions' => array(
                        'select' => array(
                            'template' => $this->getTemplateType('browse_select_button')
                        ),
                    )
                ));
        } else {
            $listMapper
                ->addIdentifier('getPathFile', null, ['label' => 'Image',
                    'template' => $this->getTemplateType('image')
                ])
                ->add('name', null, ['label' => 'Titre','editable' => true]);
            $listMapper
                ->add('_action', 'actions', $this->getDefaultListActionParams());
        }
    }


    protected function configureFormFields(FormMapper $formMapper)
    {
        $isUpdate = $this->isUpdate();

        $formMapper
            ->with("Fichier", ['class' => 'col-md-3','translation_domain' => $this->baseTranslationDomain])
            ->add('pathFile', FileType::class, [
                'required' => !$isUpdate,
                'label' => 'Image',
                'preview_property' => 'getPathFile'
            ])
            ->end()
            ->with("Descriptifs de l'image", ['class' => 'col-md-9','translation_domain' => $this->baseTranslationDomain])
            ->add(
                'translations2',
                TranslatableType::class,
                [
                    'fields' => [
                        'name' => [
                            'field_type' => TextType::class,
                            'label' => 'Titre',
                            'required' => false
                        ],
                        'alt' => [
                            'field_type' => TextareaType::class,
                            'label' => 'Texte alternatif',
                            'required' => false
                        ]
                    ]
                ]
            )
            ->end();
    }

    /**
     * @param mixed $object
     * @return string
     */
    public function toString($object)
    {
        return $object instanceof Image
            ? '<img src="/'.$object->getPathFile().'" style="max-height:30px;max-width:100px"/>'
            : 'Image'; // shown in the breadcrumb on the create view
    }

    /**
     * @param RouteCollection $collection
     */
    protected function configureRoutes(RouteCollection $collection)
    {
        parent::configureRoutes($collection);
        $collection->add('browse', 'browse');
        $collection->add('upload', 'upload');
        $collection->remove('clone');
    }

    /**
     * @param string $action
     * @param null $object
     * @return array
     */
    public function getActionButtons($action, $object = null)
    {
        $actions = parent::getActionButtons($action, $object); // TODO: Change the autogenerated stub

        if (in_array($action, array('browse', 'list'))
            && $this->hasAccess('create')
            && $this->hasRoute('create')
        ) {
            if ($action == 'browse') {
                $actions = array();
            }
            $actions['upload'] = array(
                'template' => $this->getTemplateType('upload_button'),
            );
        }

        return $actions;
    }

    /**
     * @param ErrorElement $errorElement
     * @param mixed $object
     */
    public function validate(ErrorElement $errorElement, $object)
    {
        $errorElement
            ->with('pathFile')
            ->assertImage()
            ->end()
        ;
    }

    /**
     * @param $image
     * @return string
     */
    public function getFilePath($image)
    {
        return '/'.$image->getPathFile();
    }

    /**
     * @param $image
     * @return mixed
     */
    public function getFileAlt($image)
    {
        return $image->getAlt();
    }
}
